#
# spec file for package ghc-buffer-builder
#
# Copyright (c) 2017 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via http://bugs.opensuse.org/
#


%global pkg_name buffer-builder
%bcond_with tests
Name:           ghc-%{pkg_name}
Version:        0.2.4.4
Release:        lp150.1.8
Summary:        Library for efficiently building up buffers, one piece at a time
License:        BSD-3-Clause
Group:          Development/Libraries/Haskell
URL:            https://hackage.haskell.org/package/%{pkg_name}
Source0:        https://hackage.haskell.org/package/%{pkg_name}-%{version}/%{pkg_name}-%{version}.tar.gz
BuildRequires:  gcc-c++
BuildRequires:  ghc-Cabal-devel
BuildRequires:  ghc-bytestring-devel
BuildRequires:  ghc-mtl-devel
BuildRequires:  ghc-rpm-macros
BuildRequires:  ghc-text-devel
BuildRequires:  ghc-unordered-containers-devel
BuildRequires:  ghc-vector-devel
BuildRequires:  libstdc++-devel
%if %{with tests}
BuildRequires:  ghc-HTF-devel
BuildRequires:  ghc-aeson-devel
BuildRequires:  ghc-attoparsec-devel
BuildRequires:  ghc-criterion-devel
BuildRequires:  ghc-deepseq-devel
BuildRequires:  ghc-quickcheck-instances-devel
%endif

%description
"Data.BufferBuilder" is an efficient library for incrementally building up
'ByteString's, one chunk at a time. Early benchmarks show it is over twice as
fast as ByteString Builder, primarily because 'BufferBuilder' is built upon an
ST-style restricted monad and mutable state instead of ByteString Builder's
monoidal AST.

Internally, BufferBuilder is backed by a few C functions. Examination of GHC's
output shows nearly optimal code generation with no intermediate thunks -- and
thus, continuation passing and its associated indirect jumps and stack traffic
only occur when BufferBuilder is asked to append a non-strict ByteString.

I benchmarked four approaches with a URL encoding benchmark:

* State monad, concatenating ByteStrings: 6.98 us

* State monad, ByteString Builder: 2.48 us

* Crazy explicit RealWorld baton passing with unboxed state: 28.94 us (GHC
generated really awful code for this, but see the revision history for the
technique)

* C + FFI + ReaderT: 1.11 us

Using BufferBuilder is very simple:

> import qualified Data.BufferBuilder as BB > > let byteString =
BB.runBufferBuilder $ do > BB.appendBS "http" > BB.appendChar8 '/' >
BB.appendBS "//"

This package also provides "Data.BufferBuilder.Utf8" for generating UTF-8
buffers and "Data.BufferBuilder.Json" for encoding data structures into JSON.

%package devel
Summary:        Haskell %{pkg_name} library development files
Group:          Development/Libraries/Haskell
Requires:       %{name} = %{version}-%{release}
Requires:       ghc-compiler = %{ghc_version}
Requires:       libstdc++-devel
Requires(post): ghc-compiler = %{ghc_version}
Requires(postun): ghc-compiler = %{ghc_version}

%description devel
This package provides the Haskell %{pkg_name} library development files.

%prep
%setup -q -n %{pkg_name}-%{version}

%build
%ghc_lib_build

%install
%ghc_lib_install

%check
%cabal_test

%post devel
%ghc_pkg_recache

%postun devel
%ghc_pkg_recache

%files -f %{name}.files
%doc LICENSE

%files devel -f %{name}-devel.files
%doc changelog.md

%changelog
* Thu Aug  3 2017 psimons@suse.com
- Updated with latest spec-cleaner version 0.9.8-8-geadfbbf.
* Sun Feb 12 2017 psimons@suse.com
- Update to version 0.2.4.4 with cabal2obs.
